import React, { useState, useEffect, useMemo, Fragment } from 'react'
import styles from './index.less'
import {  Input,  Popconfirm, Select , Button , Space } from 'antd'
import Drag from './../../../../components/Drag'
import DragTable from '../../../../components/DragTable'
import { DeleteOutlined, MenuOutlined , SettingOutlined } from '@ant-design/icons';
import * as antdIcons from "@ant-design/icons";
import { colors } from '../../../../../constant'
import { getNanoid } from '../../../../../utils'

const getIconsList = () => {
  const arr = [];

  Object.keys(antdIcons).forEach(key => {
    const Icon = antdIcons[key];

    if(key !== "default" &&
      key !== "IconProvider" &&
      key !== "setTwoToneColor" &&
      key !== "getTwoToneColor" &&
      key !== "createFromIconfontCN"){
      arr.push({
        label: <Icon />,
        value: key,
        key,
      })
    }

  })

  return arr;
}


const OptionRender = (props) => {
  const { value, onChange, addons , schema} = props
  const { color, isMultiple, widget, } = addons.formData;
  const { name } = schema;
  const [loading, setLoading] = useState(true)
  const [currentColor, setCurrentColor] = useState('')
  const [visible , setVisible] = useState(false);
  const [url , setUrl] = useState("");

  const iconList = getIconsList();

  useEffect(() => {
    if (addons) {
      if (!isMultiple && widget === 'ComboBox') {
        addons.setValue('color', false)
      }
    }
  }, [isMultiple])

  useEffect(() => {
    setTimeout(() => {
      setLoading(false)
    }, 0)
  }, [])

  const onPressEnter = (i) => {
    let length = [...value].length
    let array = [...value]
    array.splice(i + 1, 0, { key: getNanoid(), index: length + 1, color: colors[length % 10], value: '' })
    onChange(array)
  }

  const inputChange = (item, i , key) => {
    let array = []
    value.forEach((v, index) => {
      if (index === i) {
        v[key] = item;
      }
      array.push({ ...v })
    })

    onChange(array)
  }

  const confirm = (index) => {
    let array = []
    value.forEach((v, i) => {
      if (i !== index) {
        array.push({ ...v })
      }
    })
    onChange(array)
  }

  const colorClick = (item, i) => {
    let array = []
    value.forEach((v, index) => {
      if (index === i) {
        v.color = item
      }
      array.push({ ...v })
    })
    onChange(array)
  }

  const rest = (index) => {
    let array = []
    value.forEach((v, i) => {
      if (index === i) {
        v.color = colors[index % 10]
      }
      array.push({ ...v })
    })
    onChange(array)
  }

  const columns = useMemo(() => {
    let columns = [
      {
        title: '显示类型',
        dataIndex: 'type',
        width: 60,
        render: (r, _, i) => {
          return (
            <Select options={[
                      {
                        label: "文字",
                        value: "text"
                      },
                      {
                        label: "图标",
                        value: "icon"
                      },
                      {
                        label: "超链接",
                        value: "link"
                      }
                    ]}
                    value={r}
                    style={{
                      width: "100%"
                    }}
                    onChange={(e) => inputChange(e, i , "type")} />
          )
        }
      },
      {
        title: '显示名称',
        dataIndex: 'value',
        width: 100,
        render: (r, _, i) => {
          return (
            <Fragment>
              {
                _.type === "icon" ? (
                  <Select options={iconList}
                          value={r}
                          style={{
                            width: "100%"
                          }}
                          allowClear
                          showSearch
                          onChange={(e) => inputChange(e, i, "value")} />
                ) : (
                  <Input
                    onPressEnter={() => onPressEnter(i)}
                    onChange={(e) => inputChange(e.target.value, i, "value")}
                    value={r}
                  />
                )
              }

            </Fragment>

          )
        }
      },
      {
        title: '动作',
        dataIndex: 'action',
        width: 100,
        render: (r, _, i) => {
          return (
            <Fragment>
              <Select options={[
                {
                  label: "删除",
                  value: "delete"
                },
                {
                  label: "编辑",
                  value: "update"
                },
                {
                  label: "跳转",
                  value: "href"
                }
              ]} value={r}
                 style={{
                   width: 80
                 }}
                 onChange={(e) => inputChange(e, i, "action")} />

              {
                r === "href" && (
                  <Button type={"text"}
                          onClick={() => {
                            setUrl(_.url);
                            setVisible(true);
                          }}
                          icon={<SettingOutlined />} />
                )
              }
            </Fragment>
          )
        }
      },
      // {
      //   title: '颜色',
      //   dataIndex: 'color',
      //   width: 30,
      //   render: (r, _, index) => {
      //     const content = (
      //       <div>
      //         <div className={styles.colorTop}>
      //           <div className={styles.topLeft}>请选择颜色</div>
      //           <div className={styles.topRight}>
      //             <Popconfirm
      //               overlayClassName={styles.Popconfirmtitle}
      //               placement="topLeft"
      //               icon={false}
      //               title={
      //                 <SketchPicker width="217px" color={currentColor} onChange={e => {
      //                   setCurrentRgba(`rgb(${e.rgb.r}, ${e.rgb.g}, ${e.rgb.b})`)
      //                   setCurrentColor(e.hex)
      //                 }} />
      //               }
      //               onConfirm={() => {
      //                 let array = []
      //                 value.forEach((v, i2) => {
      //                   if (index === i2) {
      //                     v.color = currentRgba
      //                   }
      //                   array.push({ ...v })
      //                 })
      //                 onChange(array)
      //               }}
      //             >
      //               <div className={styles.colorsSelect}></div>
      //             </Popconfirm>
      //           </div>
      //         </div>
      //         <div className={styles.colorBox}>
      //           {
      //             colors.map((v, i) => {
      //               return (
      //                 <div className={styles.colors} style={{ background: v }} onClick={() => colorClick(v, index)}>
      //                   <CheckOutlined style={{ fontSize: '11px', marginLeft: '5px', display: r === v ? 'inline-block' : 'none' }} />
      //                 </div>
      //               )
      //             })
      //           }
      //         </div>
      //       </div>
      //     )
      //     return (
      //       <Popover
      //         content={content}
      //         trigger='click'
      //       >
      //         <div className={styles.colors} style={{ background: r || '#f2f4f5' }}></div>
      //       </Popover>
      //     )
      //   }
      // },
      {
        width: 30,
        render: (r, _, i) => {
          return (
            <Fragment>
              <Popconfirm
                title='确定删除?'
                onConfirm={() => confirm(i)}
                onCancel={() => { }}
              >
                <DeleteOutlined style={{ color: 'rgb(153, 153, 153)' }} />
              </Popconfirm>
            </Fragment>

          )
        },
      },
      {
        title: '',
        dataIndex: 'sort',
        width: 30,
        className: 'drag-visible',
        render: () => <MenuOutlined style={{ cursor: 'grab', color: '#999', }} />,
      },
    ]
    if (!color) {
      return columns.filter(v => v.dataIndex !== 'color')
    }
    return columns
  }, [value, color, currentColor])

  const switchChange = (checked) => {
    addons.setValue('color', checked)
    if (checked) {
      if (Array.isArray(value)) {
        let array = []
        value.forEach((v, i) => {
          array.push({ ...v, color: colors[i % 10] })
        })
        onChange(array)
      }
    }
  }

  const add = () => {
    let length = [...value].length
    let array = [...value]
    array.push({ key: getNanoid(), index: length + 1, color: colors[length % 10], value: '' })
    onChange(array)
  }

  const onOk = () => {
    const list = [...value];
    list.forEach(item => {
      if(item.type === "link"){
        item.url = url
      }
    });

    onChange(list);
    setVisible(false);
  }

  const dragCallBack = (array) => {
    if (array) {
      onChange(array)
    }
  }

  return (
    <div className={styles.simpleList}>
      <div className={styles.title}>
        <div style={{ fontSize: '14px' }}>{name}</div>
        {/*<div style={{ fontSize: '12px' }}>*/}
        {/*  <span style={{ paddingRight: '5px' }}>彩色</span>*/}
        {/*  <Switch size="small" checked={color} onChange={switchChange} />*/}
        {/*</div>*/}
      </div>
      <div className={styles.box}>
        {loading ? null : (
          <DragTable
            dataSource={
              Array.isArray(value)
                ? value.map((v, i) => {
                    return { key: `${i}`, index: i, ...v };
                  })
                : []
            }
            showHeader={true}
            rowKey="key"
            ItemTypes="stadingOrder"
            color={color}
            scroll={{
              x: 500,
            }}
            columns={columns}
            pagination={false}
            dragCallBack={dragCallBack}
          />
        )}
      </div>
      {value?.length < 3 && (
        <div className={styles.footer}>
          <span onClick={add}>添加选项</span>
        </div>
      )}

      <Drag title="跳转链接"
            onOk={onOk}
            onCancel={() => setVisible(false)}
            visible={visible}>
        <Input.TextArea rows={5} value={url} placeholder="请输入跳转链接" onChange={e => setUrl(e.target.value)} />
      </Drag>
    </div>
  );
}

export default OptionRender
